import 'package:meta/meta.dart';

import 'constants.dart';

@immutable
class Angle {
  final double _degrees;
  const Angle.fromDeg(num degrees)
      : _degrees = (degrees + 180.0) % 360.0 < 0.0
            ? (degrees + 180.0) % 360.0 + 180.0
            : (degrees + 180.0) % 360.0 - 180.0;
  const Angle.fromRad(num radians)
      : _degrees = (radians * radToDeg + 180.0) % 360.0 < 0.0
            ? (radians * radToDeg + 180.0) % 360.0 + 180.0
            : (radians * radToDeg + 180.0) % 360.0 - 180.0;
  const Angle.nan() : _degrees = double.nan;
  const Angle.infinity() : _degrees = double.infinity;
  const Angle.zero() : _degrees = 0.0;

  /// Returns a [int] hash code for a numerical value of [Angle] instance.
  @override
  int get hashCode => _degrees.hashCode;

  /// Returns [Angle]'s value in degrees
  double get deg => _degrees;

  /// Returns [Angle]'s value in radians
  double get rad => _degrees * degToRad;

  bool get isNaN => _degrees.isNaN;

  /// String representation of the value of [Angle]
  String get toStringDeg => '${_degrees.toStringAsFixed(1)}°';

  /// String representation of the instance of [Angle]
  String toString() => 'Angle: ${_degrees.toStringAsFixed(1)}°';

  @override
  bool operator ==(Object other) =>
      other is Angle && _degrees == other._degrees;
  bool operator >(Angle other) => _degrees > other._degrees;
  bool operator >=(Angle other) => _degrees >= other._degrees;
  bool operator <(Angle other) => _degrees < other._degrees;
  bool operator <=(Angle other) => _degrees <= other._degrees;
  Angle operator +(Angle other) => Angle.fromDeg(_degrees + other._degrees);
  Angle operator -(Angle other) => Angle.fromDeg(_degrees - other._degrees);
  Angle operator *(num other) => Angle.fromDeg(_degrees * other);
  Angle operator /(num other) => Angle.fromDeg(_degrees / other);
}
